#include "irensei_rule_description.h"
#include "irensei_ai_descripter.h"
#include "game.h"
#include <stdlib.h>
#include <stdio.h>
#include <windows.h>
#include <assert.h>


using namespace Irensei;

//																		//
//									AI									//

class AI {
public:
	AI( const IRENSEI_RULE_DESCRIPTION & rule, Game::EBlackOrWhite side );

	int Think( IRENSEI_BOARD_POSITION & result, const IRENSEI_BOARD_POSITION * lastMove, HANDLE hout );

	int Undo( IRENSEI_BOARD_POSITION & lastMove, HANDLE hout );

	int Debug( IRENSEI_DEBUG_BOARD & dest, int idx, HANDLE hout );
	static int CountDebug( void )		{ return 1; };

private:
	AI();		// ftHgRXgN^𖳌

	Game game;
	Game::EBlackOrWhite myside;

	const IRENSEI_RULE_DESCRIPTION rule;

	IRENSEI_DEBUG_BOARD dbBoard;
};

AI::AI( const IRENSEI_RULE_DESCRIPTION & rl, Game::EBlackOrWhite side )
		:
	myside(side),
	rule(rl)
{
	game.SetRule( rule );
}

int AI::Think( IRENSEI_BOARD_POSITION & result, const IRENSEI_BOARD_POSITION * const lastMove, HANDLE hout )
{
	if ( lastMove )
		game.NewMove( lastMove->x, lastMove->y );

	memset( dbBoard.stones, -128, sizeof(dbBoard.stones) );
	for ( int y = rule.outerInvalidLines; y < 19-rule.outerInvalidLines; y++ ) {
		for ( int x = rule.outerInvalidLines; x < 19-rule.outerInvalidLines; x++ ) {
			Game::EMoveResult legality = game.CheckLegalMove( game.GetBoard(), x, y, myside );
			dbBoard.stones[y][x] = legality;
			if ( Game::EMResultWin >= legality ) {
				result.x = x;
				result.y = y;
				game.NewMove( x, y );
				return 1;
			}
		}
	}

	// 
	return 0;
}

int AI::Undo( IRENSEI_BOARD_POSITION & lastMove, HANDLE hout )
{
	assert( 0 != game.CountMove() );
	game.Undo();

	const Game::Move lm = game.GetLastMove();
	lastMove.x = lm.x;
	lastMove.y = lm.y;

	return 1;
}

int AI::Debug( IRENSEI_DEBUG_BOARD & dest, int idx, HANDLE hout )
{
	if ( idx != 0 )
		return 0;

	memcpy( &dest, &dbBoard, sizeof(dest) );
	return 1;
}


//																		//
//						DLL C^[tF[X							//

extern "C" __declspec(dllexport) int IrenseiGetName( char dest[IRENSEI_AI_NAME_LEN] )	{ strcpy( dest, "lefttop(cl sample)" ); return 0; }
extern "C" __declspec(dllexport) int IrenseiGetFirstVersion( void )		{ return 100; }
extern "C" __declspec(dllexport) int IrenseiGetLastVersion( void )		{ return 100; }

extern "C" __declspec(dllexport) int IrenseiInitialize(const IRENSEI_RULE_DESCRIPTION * rule, int side, int hout)
{
	if ( ! CheckIrenseiRuleDescriptionValidity( rule ) )
		return 0;

	srand(GetTickCount());

	IrenseiPrintf( reinterpret_cast<HANDLE>(hout), "lefttop(cl) AI initialized.\n" );
	return reinterpret_cast<int>( new AI( *rule, static_cast<Game::EBlackOrWhite>(side) ) );
}

extern "C" __declspec(dllexport) int IrenseiFinalize(int param, int hout)
{
	delete reinterpret_cast<AI*>( param );
	IrenseiPrintf( reinterpret_cast<HANDLE>(hout), "lefttop(cl) AI finalized.\n" );
	return 1;
}

extern "C" __declspec(dllexport) int IrenseiThink(int param, IRENSEI_BOARD_POSITION * result, const IRENSEI_BOARD_POSITION * lastMove, int hout)
{
	return reinterpret_cast<AI*>( param )->Think( *result, lastMove, reinterpret_cast<HANDLE>(hout) );
}

extern "C" __declspec(dllexport) int IrenseiUndo( int param, IRENSEI_BOARD_POSITION * lastMove, int hout )
{
	return reinterpret_cast<AI*>( param )->Undo( *lastMove, reinterpret_cast<HANDLE>(hout) );
}

extern "C" __declspec(dllexport) int IrenseiDebug( int param, IRENSEI_DEBUG_BOARD * dest, int idx, int hout )
{
	return reinterpret_cast<AI*>( param )->Debug( *dest, idx, reinterpret_cast<HANDLE>(hout) );
}

extern "C" __declspec(dllexport) int IrenseiDebugCount( void )
{
	return 1;
}

